#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define XSIZE 100 // Maximum X size of a map
#define YSIZE 100 // Maximum Y size of a map

int main (int argc, char *argv[])
{
	char map[56][100]; // this will hold the blank hex map
	char mapcopy[56][100]; // temporary copy of hex map
	char tc; // target character
	char temp[256]; // temporary string variable
	char temp1[16],temp2[16]; // two small temporary strings
	int i,j,k,l,m,n,o,p; // temporary integers, mostly looping
	int sx,sy;
	int x,y; // x and y size of actual image being loaded
	int xsector,ysector; // "quadrant" currently being printed
	char inp_file[13]; // input file name
	char out_file[13]; // output file name stub
	char hex_file[13]; // name of the hex file
	int elev[YSIZE][XSIZE]; // elevations for map
	char terr[YSIZE][XSIZE]; // terrain for map
	FILE *in,*out,*hex; // file pointers

	if (argc!=3)
	{
		printf("Please supply the name of a file generated by MAPMAKER, along with an\n");
		printf("output file to be generated by MAPPRINT.\n");
		return -1;
	}

	strcpy(inp_file,argv[1]); // get input file name
	strcpy(out_file,argv[2]); // get output file name
	strcpy(hex_file,"hex.txt\0"); // we already know the hex file's name

	if ((hex=fopen(hex_file,"r"))==NULL) // load hex map for reading
	{
		printf("Could not open HEX.TXT for read access.\n");
		return -3;
	}

	for (i=0;i<56;i++)
		fgets(map[i],100,hex); // reads in the hex map, including newline character
	fclose(hex);

	if ((in=fopen(inp_file,"r"))==NULL) // open input file for reading
	{
		printf("Could not open input file for read access.\n");
		return -2;
	}

	fgets(temp,256,in); // first line of input file is x and y size
	x=0;
	y=0;
	for (k=0;k<16;k++) // blank out temporary storage
	{
		temp1[k]='\0';
		temp2[k]='\0';
	}
	for (i=0;temp[i]==' ';i++); // returns beginning of x string
	for (j=i;temp[j]!=' ';j++); // returns end of x string
	for (k=i;k<j;k++) temp1[k-i]=temp[k]; // copy x string to temp1
	for (i=j;temp[i]==' ';i++); // returns beginning of y string
	for (j=i;((temp[j]!=' ')&&(temp[j]!='\n'));j++); // returns end of y string
	for (k=i;k<j;k++) temp2[k-i]=temp[k]; // copy y string to temp2
	x=atoi(temp1); // convert temp1 to integer form
	y=atoi(temp2); // convert temp2 to integer form
	for (k=0;k<16;k++) // blank out temporary storage again
	{
		temp1[k]='\0';
		temp2[k]='\0';
	}
	for (i=0;i<y;i++) // time to read terrain and elevation
	{
		fgets(temp,256,in); // get a line
		for (j=0;j<x;j++) // terrain/elevation are pairs
		{
			terr[i][j]=temp[(j*2)]; // first char goes to terrain
			temp1[0]=temp[((j*2)+1)]; // second char goes to elevation
			elev[i][j]=atoi(temp1); // convert temp1 to integer
		}
	}
	fclose(in);
	// time to process and make map!
	// map sheets will be made from left to right working from
	// top to bottom.  Only a 6x6 area can be printed on a single
	// 8.5x11 so multiple files will be created (or, I could put in
	// a FORMFEED char in between sections?

	if ((out=fopen(out_file,"w"))==NULL)
	{
		printf("Unable to open output file for write access.\n");
		return -4;
	}

	printf("Beginning processing, please stand by.\n");

	for (i=0;i<y;i+=6) // break it vertically into chunks of 6
	{
		ysector=(i / 6); // compute y sector

		for (j=0;j<x;j+=6) // break it horizontally into chunks of 6
		{
			xsector=(j / 6); // compute x sector

			for (k=0;k<56;k++)
				strcpy(mapcopy[k],map[k]); // take a fresh copy of the map

			for (k=0;k<6;k++)
			{
				itoa(((ysector*6)+k),temp1,10);
				itoa(((xsector*6)+k),temp2,10);
				printf("Processing hex %s, %s\n",temp2,temp1);
				m=strlen(temp1);
				for (n=0;n<m;n++)
					mapcopy[9+(k*8)][n]=temp1[n]; // add row reference #'s
				m=strlen(temp2);
				for (n=0;n<m;n++)
					mapcopy[0][9+(k*12)+n]=temp2[n]; // add column reference #'s
			}

			itoa(xsector,temp1,10); // add the sector reference
			itoa(ysector,temp2,10);
			k=strlen(temp1);
			for (l=0;l<k;l++) // this adds the x
				mapcopy[55][9+l]=temp1[l];
			m=l;
			mapcopy[55][9+m]=','; // this adds a comma and a space
			m++;
			mapcopy[55][9+m]=' ';
			m++;
			k=strlen(temp2);
			for (l=0;l<k;l++) // and this adds the y
				mapcopy[55][9+m+l]=temp2[l];
			mapcopy[55][9+m+l]='\0';

			for (k=0;((k<6)&&(((ysector*6)+k)<y));k++) // step through the y sectors
			{

				for (l=0;((l<6)&&(((xsector*6)+l)<x));l++) // step through the x sectors
				{
					sx=2+(l*12);
					sy=2+(k*8)+(4*(l % 2));
					if (l % 2)
						tc='b';
					else
						tc='a';
					// nothing here yet!  but this is where important stuff goes
					for (m=0;m<8;m++) // time to put the terrain in
						for (n=0;n<16;n++)
							if (mapcopy[sy+m][sx+n]==tc)
								mapcopy[sy+m][sx+n]=terr[i+k][j+l]; // copy out terrain
					itoa((elev[i+k][j+l]),temp1,10);
					m=strlen(temp1);
					for (n=0;n<m;n++)
						mapcopy[sy+4][sx+7+n]=temp1[n]; // copy out elevation
				}
			}
			// okay, clean up any stray a's and b's
			for (k=0;k<56;k++)
				for (l=0;l<100;l++)
					if ((mapcopy[k][l]=='a')||(mapcopy[k][l]=='b')||(mapcopy[k][l]=='#'))
						mapcopy[k][l]=' ';
			// write the new map sector to the output file and add a FF
			for (k=0;k<56;k++)
				fprintf(out,"%s",mapcopy[k]);
				fprintf(out,"%c",0x0C);
				fprintf(out,"%c",'\n');
		}
	}
	fclose(out);
	printf("Processing is complete.\n");
	return 1;
}
