/*  This is a very crude program written for use with the BattleTech 3056
	 MUSE.  By running this program, a user may generate a random map of
	 up to 500 x 500 hexes (or 15km by 15km).

MAPMAKER version 1.0b by Ted Wallace

NOTE: THIS IS A BETA VERSION.  PLEASE USE THIS PROGRAM AND SEND COMMENTS 
AND SUGGESTIONS FOR ITS IMPROVEMENT TO THE AUTHOR AT: falryx@dartmouth.edu
YOUR USE AND INPUT WILL HELP TO MAKE THIS PROGRAM AS USEFUL AS POSSIBLE.

==== INSTRUCTIONS ====

	 First: compile the program.  A simple 'make mapmaker' should do.  Worse
	 comes to worse, you can use 'cc -o mapmaker mapmaker.c' and this should
	 do the job.

	 One compiled you start the program by typing in the program's name and
	 the mapsize at your command prompt (i.e.):
	 % mapmaker 20 20
	 Would begin the program to create a 20 hex by 20 hex map.

	 The program will initialize the map (placing clear terrain in all the
	 hexes) then begin to prompt you for data.  Most of the questions are
	 self-explanatory.  Until I replace integers with characters, all 
	 yes/no questions can be answered with '1' (yes) or '0' (no).

	 After prompting for the number of rivers and lakes, the program will
	 move into 'make lakes mode'.  It allows you to place lakes manually
	 or randomly.  After determining the position of the lake, it asks
	 for an initial depth (this can fluctuate up and down) and a 'growth'
	 rate.  This rate is used to govern the size of the lake.  High 
	 growth rates mean large lakes.  Small rates have smaller sized lakes.
	 This rate has no effect on relative depth - other than the further 
	 from the initial hex, the more likely the lake will be more shallow.
	 I've found that a 'growth' of about 100 creates a decent sized lake.


	 Once all the lakes have been made it prompts for manual or random
	 entry of river starting positions.  After determining the position
	 you will be asked on a relative scale, how long the river should be.
	 A '100' percent will ensure that the river will continue going until
	 it reaches SOME edge of the map (although not necessarily the one you
	 intend).  After this, you give the general direction of the river, and
	 it digs itself.

	 Your next job is to 'seed' forest hexes.  For the sake of providing
	 reference the program lets you know how many hexes you have total.
	 If you don't want the program to seed hexes randomly (i.e. you want
	 to place all the forests manually) enter a 0 here.

	 Forests have a 'growth' rate similar to lakes.  Once more, a growth of
	 about 100 creates a decent sized forest.

	 The next job is the same as the previous one, except with the 'hill'
	 terrain.  The questions are the exact same, except that you must also
	 enter in an initial height.  This basically builds an upside-down
	 lake and uses the '^' terrain rather than the '~' terrain.  

	 Lastly, you can add other elevation features, such as a ridge, a 
	 plateau, or a 'rise' (it's actually a conical rise in elevation).
	 This last part does not modify the terrain at all -- ONLY the elevation
	 is modified.  This is the command to use if you want your forest to have
	 a plateau or ridge in it.

	 Ridges run in a straight line at the specified height for the specified
	 length from the specified starting position.  This is done so that you
	 may either have several ridges back to back (creating a 'ramp' effect)
	 or just a single ridge.

	 Plateaus are currently not coded. (although they are next).

	 Rises are currently not coded.
	

==== COMMENTS FROM THE AUTHOR ====

	 Things to do down the road:

	 * Tinker a little with different growth rates and find a good range 
	 for appropriate limitations for a more 'natural' feel.

	 * Try to regionalize terrain so there isn't such heavy variation.
	 A lot of work has gone into making this not completely random (it
	 looks hideous if it is -- trust me), however there is still a little
	 too much terrain variation from hex to hex.  Working on a more 
	 macroscopic scale could improve this.  [this is the next major revision
	 I will do]

	 You'll note I have a lot of 'redundant' functions which I could have
	 placed into one generic one for generation of terrain.  This would be
	 a lot more OO and all-in-all a better design, but I specifically wanted
	 to keep them separate to account for different trends in forest, hill
	 and lake growth -- if I wanted to modify one without changing all three.
	 This makes the program longer and more difficult to read, but in the
	 end provides me with a little more flexibility.

==== HISTORY ====

This program was initially written using K&R C on a Sun SPARCstation IPC
at the Maria Mitchell Observatory on Nantucket, MA.

October 1993   :  Program first concieved and worked upon by Ted Wallace.  
						His comment: "It sucked."

Novemeber 1993 :  Major rewrite of entire program by Ted Wallace.
  Nov. 11, 1993:  Finished with 'water' placement.  The process is 0w 
						more interactive and less random, allowing the player to
						define parameters which govern the shape and size of any
						body of water.  This creates more believeable work than 
						a randomized placement of water.

						Began work on forest placement in the same semi-interactive
		  vein.

  Nov. 12, 1993:  Finished forest placement work.  The process allows for
						both random seeding and interactive placement of forests.
		  The user may either randomly generate growth rates (and
		  therefore size of the forest) or manually enter them in.
		  For manually placed forests, the growth rate must be
		  manually entered.

		  Finished work on hill placement.  Like forestry, the 
		  process allows for both random seeding and interactive
		  placement. (Likely, lakes will be updated to this as
		  well).

		  Began work on elevation stuff for the creation of 
		  other little 'bumps' elevation.

  Nov. 18, 1993:  Fixed a bug in forge_river() which was jumping hexes
						inappropriately (as was apparent from a 100 x 100 map
		  generated on the 13th).

		  Wrote make_ridge().  Ridge generation is now complete,
		  although not as robust as I would personally like.

		  Began work on make_plateau().

		  Wrote the 'Instructions' area.

		  Released version 1.0b for general use and user-feedback.

==== CREDITS =====

This program originally written by Ted Wallace (a.k.a. Falryx) reachable
via Internet email at 'falryx@dartmouth.edu' or via telnet at any one of
several major MUSEs or MOOs.

This program is in Beta release and currently freely distributable.  I
ask that no modifications be made to the program until a 'final' release
of version 1.0.  Thereafter modifications which are made should please
include a credit to me -- even if you wipe out all my simple code.
Suggestions for improvement can be sent to me, and I'll be happy to include
them (if they are, in fact, improvements) with a credit to you as author.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

#define MAXSIZE 100               /* Maximum dimensions possible for the map */

  struct element {
	 char terrain;
	 int height;
  };

struct element map[MAXSIZE][MAXSIZE];
/* struct element near[6]; */
int xsize;
int ysize;

void initialize_map()
{
  int xc, yc;

  printf("MAPMAKER: Initializing map grid.  Size: %d x %d.\n", xsize, ysize);
  for (yc = 0; yc < ysize; yc++)
	 for (xc = 0; xc < xsize; xc++) {
		map[xc][yc].terrain = '.';
		map[xc][yc].height  = 0;
	 }
}


void dig_um(x, y, d, g)
int x, y, d, g;
{
  char t;
  int growth, depth, d2;

  growth = depth = 0;
  t = '~';
  
  if (x >= 0 && x < xsize && y >=0 && y < ysize)
	 if (map[x][y].terrain != t && map[x][y].terrain != 'E') {
		if (d > 1 || (rand() % 100) <= g) {
	d2 = d + (rand() % 5 - 2);
	d2 = (d2 > 9) ? 9 : d2;
	d2 = (d2 < 1) ? 1 : d2;
	map[x][y].terrain = t;
	map[x][y].height =  d2;
		}
		else {
	d2 = 10;
	map[x][y].terrain = 'E';
	map[x][y].height = d2;
		}
		growth = 3 * g / 4;
		depth = d - 1;
		if (depth && d2 > 0 && d2 < 10) {
	dig_um(x, y + 1, depth, growth);
	dig_um(x, y - 1, depth, growth);
	dig_um(x + 1, y, depth, growth);
	dig_um(x - 1, y, depth, growth);
	if (x % 2) {
	  dig_um(x + 1, y - 1, depth, growth);
	  dig_um(x - 1, y - 1, depth, growth);
	}
	else {
	  dig_um(x + 1, y + 1, depth, growth);
	  dig_um(x - 1, y + 1, depth, growth);
	}
		}
	 }
}

void dig_lake(x, y)
int x, y;
{
  int depth;
  int growth;
  char t;

  t = '~';
  depth = growth = 0;

  printf("MAPMAKER: Digging lake at (%d, %d)\n", x, y);
  while (depth < 1 || depth > 9) {
	 printf("          What should the initial depth of this lake be? [1-9] ");
	 scanf("%d", &depth);
  }
  printf("          What should growth rate of the lake be? ");
  scanf("%d", &growth);
 
  map[x][y].terrain = t;
  map[x][y].height = depth;

  dig_um(x, y - 1, depth, growth);
  dig_um(x + 1, y, depth, growth);
  dig_um(x - 1, y, depth, growth);
  if (x % 2) {
	 dig_um(x + 1, y - 1, depth, growth);
	 dig_um(x - 1, y - 1, depth, growth);
  }
  else {
	 dig_um(x + 1, y + 1, depth, growth);
	 dig_um(x - 1, y + 1, depth, growth);
  }

  printf("MAPMAKER: Lake dug.\n");
}

void forge_river(x, y, fx, fy, p, d)
int x, y;
int fx, fy;
int p, d;
{
  int r1, r2;
  int dx, dy;

  r2 = rand() % 100;
  if (r2 <= p) {
	 dx = fx;
	 dy = fy;

	 while (dx == fx && dy == fy) {
		if (d == 1) {
	dx = x + (rand() % 3 - 1);
	if (dx == x) 
	  dy = y + ((rand() % 3) ? -1 : 1);
	else if (x % 2)
	  dy = y + ((rand() % 3) ? 0 : -1);
	else
	  dy = y + ((rand() % 3) ? 1 : 0);
		}
		else if (d == 2) {
	r1 = rand() % 4;
	dx = x + (r1 < 0 ? -1 : (r1 > 0 ? 1 : 0));
	if (x == dx)
	  dy = y + (rand() % 2 ? -1 : 1);
	else if (x % 2)
	  dy = y + (rand() % 2 ? -1 : 0);
	else
	  dy = y + (rand() % 2 ? 0 : 1);
		}
		else if (d == 4) {
	r1 = rand() % 4;
	dx = x + (r1 < 0 ? 1 : (r1 > 0 ? -1 : 0));
	if (x == dx)
	  dy = y + (rand() % 2 ? -1 : 1);
	else if (x % 2)
	  dy = y + (rand() % 2 ? -1 : 0);
	else
	  dy = y + (rand() % 2 ? 0 : 1);
		}
		else {
	dx = x + (rand() % 3 - 1);
	if (dx == x)
	  dy = y + ((rand() % 3) ? 1 : -1);
	else if (x % 2)
	  dy = y + ((rand() % 3) ? -1 : 0);
	else
	  dy = y + ((rand() % 3) ? 0 : 1);
		}
	 }
	 
	 if (!(dx < 0 || dx >= xsize || dy < 0 || dy >= ysize)) {
		map[dx][dy].terrain = '~';
		map[dx][dy].height = 1;
		forge_river(dx, dy, x, y, p, d);
	 }
  }
}

void start_river(x, y, p, d)
int x, y;
int p, d;
{
  int dx, dy;
  int i;
  int oldx, oldy;
 
  dx = x;
  dy = y;
 
  for (i = 1; i <=4; i++) {
	 oldx = dx;
	 oldy = dy;
	 if (d == 3) {
		dx += (rand() % 3 - 1);
		if (oldx == dx)
	dy++;
		else if (!(oldx % 2))
	dy++;
	 }
	 else if (d == 1) {
		dx += (rand() % 3 - 1);
		if (oldx == dx)
	dy--;
		else if (oldx % 2)
	dy--;
	 }
	 else if (d == 2) {
		dx++;
		if (oldx % 2)
	dy -= rand() % 2;
		else
	dy += rand() % 2;
	 }
	 else {
		dx--;
		if (oldx % 2)
	dy -= rand() % 2;
		else
	dy += rand() % 2;
	 }

	 map[dx][dy].terrain = '~';
	 map[dx][dy].height = 1;
  }
 
  forge_river(dx, dy, oldx, oldy, p, d);
}

void dig_river(x, y)
int x, y;
{
  int dir, p;

  dir = p = 0;
  printf("MAPMAKER: Digging river at (%d, %d).\n", x, y);
  while (p < 1 || p > 100) {
	 printf("          What is the percentage this river will reach the edge of the\n          map? ");
	 scanf("%d", &p);
  }
  while (dir < 1 || dir > 4) {
	 printf("          Should the river flow North (1), East (2), South (3) or\n          West (4)? ");
	 scanf("%d", &dir);
  }
  start_river(x, y, p, dir);
}

void set_water(r, l)
int r, l;
{
  int answer;
  int yp, xp;
  int i;
  int xc, yc;
  
  answer = xp = yp = -1;

  if (l) {
	 printf("MAPMAKER: Creating lakes...\n");
	 while (answer < 0 || answer > 1) {
		printf("          Would you like to place lakes manually? [y=1/n=0] ");
		scanf("%d", &answer);
	 }
	 if (answer) 
		for (i = 1; i <= l; i++) {
	xp = yp = -1;
	while (xp < 0 || xp >= xsize || yp < 0 || yp >= ysize) {
	  printf("          Please enter a starting position for lake #%d (i.e. 4 4): ", i);
	  scanf("%d %d", &xp, &yp);
	}
	dig_lake(xp, yp);
		}
	 else
		for (i = 1; i <= l; i++) {
	xp = rand() % xsize;
	yp = rand() % ysize;
	dig_lake(xp, yp);
		}
  } 

  printf("MAPMAKER: Cleaning up...\n");
  for (yc = 0; yc < ysize; yc++)
	 for (xc = 0; xc < xsize; xc++)
		if (map[xc][yc].height == 10) {
	map[xc][yc].terrain = '.';
	map[xc][yc].height = 0;
		}

  answer = i = -1;

  if (r) {
	 printf("MAPMAKER: Creating rivers...\n");
	 while (answer < 0 || answer > 1) {
		printf("          Do you want to enter the starting position of the river\n          manually? [y=1/n=0] ");
		scanf("%d", &answer);
	 }
	 if (answer)
		for (i = 1; i <= r; i ++) {
	xp = yp = -1;
	while (xp < 0 || xp >= xsize || xp < 0 || xp >= ysize) {
	  printf("          Please enter a starting position for lake #%d (i.e. 5 0): ", i);
	  scanf("%d %d", &xp, &yp);
	}
	dig_river(xp, yp);
		}
	 else
		for (i = 1; i <= r; i++) {
	xp = rand() % xsize;
	yp = rand() % ysize;
	dig_river(xp, yp);
		}
  }
}

void plant_trees(x, y, g)
int x, y, g;
{
  char t;
  int growth;

  growth = 0;
  t = '"';
  
  if (x >= 0 && x < xsize && y >=0 && y < ysize)
	 if (map[x][y].terrain != t && map[x][y].terrain != '\'' && map[x][y].terrain != '~') {
		if ((rand() % 100) <= g) {
	map[x][y].terrain = t;
	map[x][y].height = 0;
		}
		else {
	map[x][y].terrain = '\'';
	map[x][y].height = 0;
		}
		growth = 3 * g / 4;
		if (growth > 5 && map[x][y].terrain != '\'') {
	plant_trees(x, y + 1, growth);
	plant_trees(x, y - 1, growth);
	plant_trees(x + 1, y, growth);
	plant_trees(x - 1, y, growth);
	if (x % 2) {
	  plant_trees(x + 1, y - 1, growth);
	  plant_trees(x - 1, y - 1, growth);
	}
	else {
	  plant_trees(x + 1, y + 1, growth);
	  plant_trees(x - 1, y + 1, growth);
	}
		}
	 }
}

void set_woods(w)
int w;
{
  int numseed;
  int i, growth;
  int answer, nument;
  int x, y, sx, sy;
  char forest;
  int xc, yc;
  int randans;

  forest = '"';
  nument = answer = growth = randans = -1;
  
  if (w) {
	 numseed = w;
	 printf("MAPMAKER: Seeding %d hexes with heavy forest.\n", numseed);
	 printf("          Do you wish manually enter growth rates? [y=1/n=0] ");
	 scanf("%d", &randans);
	 for (i = 1; i <= numseed; i++) {
		x = rand() % xsize;
		y = rand() % ysize;
		if (map[x][y].terrain != '~') {
	if (randans) {
	  printf("MAPMAKER: Starting forest at (%d, %d). What should the growth rate be?\n          [Enter 0 to abort this particular forest] ", x, y);
	  scanf("%d", &growth); 
	}
	else
	  growth = rand() % 100;
	if (growth > 0) {
	  map[x][y].terrain = forest;
	  plant_trees(x, y + 1, growth);
	  plant_trees(x, y - 1, growth);
	  plant_trees(x + 1, y, growth);
	  plant_trees(x - 1, y, growth);
	  if (x % 2) {
		 plant_trees(x + 1, y - 1, growth);
		 plant_trees(x - 1, y - 1, growth);
	  }
	  else {
		 plant_trees(x + 1, y + 1, growth);
		 plant_trees(x - 1, y + 1, growth);
	  }
	  printf("MAPMAKER: Forest planted.\n");
	}
		}
		else
	printf("MAPMAKER: Forest attempted to grow in water.  Aborted.\n");
	 }
	 printf("MAPMAKER: Automatic seeding and growth finished.\n");
  }
  else
	 printf("MAPMAKER: No automatic seeding done.\n");

  printf("          Do you wish to seed more hexes manually [y=1/n=0]? ");
  scanf("%d", &answer);
  while (answer < 0 || answer > 1) {
	 printf("          Do you wish to seed more hexes manually? ");
	 scanf("%d", &answer);
  }
  if (answer) {
	 while (nument < 0 || nument > xsize * ysize) {
		printf("          How many seeds will you plant? ");
		scanf("%d", &nument);
		if (nument > xsize * ysize)
	printf("MAPMAKER: That is more than the number of available hexes.");
	 }
	 for (i = 1;i <= nument; i++) {
		while (sx < 0 || sx >= xsize || sy < 0 || sy >= ysize) {
	printf("          Please enter a position for seed #%d: ", i);
	scanf("%d %d", &sx, &sy);
	if (map[sx][sy].height) {
	  printf("MAPMAKER: That is a water hex.");
	  sx = sy = -1;
	}
	else if (sx < 0 || sx >= xsize || sy < 0 || sy >= ysize)
	  printf("MAPMAKER: Please enter coordinates ranging from 0-%d and 0-%d\n", xsize - 1, ysize - 1);
		}
		printf("MAPMAKER: Starting forest at (%d, %d). What should the growth rate be?\n          [Enter 0 to abort this particular forest] ", sx, sy);
		scanf("%d", &growth); 
		if (growth > 0) {
	map[sx][sy].terrain = forest;
	plant_trees(sx, sy + 1, growth);
	plant_trees(sx, sy - 1, growth);
	plant_trees(sx + 1, sy, growth);
	plant_trees(sx - 1, sy, growth);
	if (x % 2) {
	  plant_trees(sx + 1, sy - 1, growth);
	  plant_trees(sx - 1, sy - 1, growth);
	}
	else {
	  plant_trees(sx + 1, sy + 1, growth);
	  plant_trees(sx - 1, sy + 1, growth);
	}
	printf("MAPMAKER: Forest planted.\n");
		}
		sx = sy = -1;
	 }
  }
}

void pile_rocks(x, y, h, g)
int x, y, h, g;
{
  char t;
  int growth, tall, t2;

  growth = tall = 0;
  t = '^';
  
  if (x >= 0 && x < xsize && y >=0 && y < ysize)
	 if (map[x][y].terrain != t && map[x][y].terrain != 'E') {
		if (h > 1 || (rand() % 100) <= g) {
	t2 = h + (rand() % 5 - 2);
	t2 = (t2 > 9) ? 9 : t2;
	t2 = (t2 < 1) ? 1 : t2;
	map[x][y].terrain = t;
	map[x][y].height =  t2;
		}
		else {
	t2 = 0;
	map[x][y].terrain = 'E';
	map[x][y].height = t2;
		}
		growth = 3 * g / 4;
		tall = h - 1;
		if (tall && t2 > 0) {
	pile_rocks(x, y - 1, tall, growth);
	pile_rocks(x, y + 1, tall, growth);
	pile_rocks(x + 1, y, tall, growth);
	pile_rocks(x - 1, y, tall, growth);
	if (x % 2) {
	  pile_rocks(x + 1, y - 1, tall, growth);
	  pile_rocks(x - 1, y - 1, tall, growth);
	}
	else {
	  pile_rocks(x + 1, y + 1, tall, growth);
	  pile_rocks(x - 1, y + 1, tall, growth);
	}
		}
	 }
}

void set_hills(h)
int h;
{
  int answer, randans, tallans;
  int growth, tall;
  char t;
  int x, y, sx, sy;
  int i, nument;
  int xc, yc;

  nument = -1;
  t = '^';
  x = y = sx = sy = -1;
  growth = tall = -1;
  answer = randans = tallans = -1;
  if (h) {
	 printf("MAPMAKER: Seeding %d hill hexes.\n", h);
	 while (randans > 1 || randans < 0) {
		printf("          Do you want their growth rates set manually? [y=1/n=0] ");
		scanf("%d", &randans);
	 }
	 while (tallans > 1 || tallans < 0) {
		printf("          Do you want their initial heights set manually? [y=1/n=0] ");
		scanf("%d", &tallans);
	 }
	 for (i = 1; i <= h; i++) {
		x = rand() % xsize;
		y = rand() % ysize;
		printf("MAPMAKER: Constructing hill #%d at (%d, %d).\n", i, x, y);
		if (randans == 1) {
	printf("          What should the growth rate of this hill be? [0 to abort] ");
	scanf("%d", &growth);
		}
		else
	growth = rand() % 100;
		if (tallans == 1) 
	while (tall < 1 || tall > 9) {
	  printf("          How tall should this hill be initially? [1-9] ");
	  scanf("%d", &tall);
	}
		else 
	tall = rand() % 9 + 1;
		if (growth && map[x][y].terrain != '~') {
	printf("MAPMAKER: Growth rate - %d, Initial height - %d.\n", growth, tall);
	map[x][y].terrain = t;
	map[x][y].height = tall;
	pile_rocks(x, y + 1, tall, growth);
	pile_rocks(x, y - 1, tall, growth);
	pile_rocks(x + 1, y, tall, growth);
	pile_rocks(x - 1, y, tall, growth);
	if (x % 2) {
	  pile_rocks(x - 1, y - 1, tall, growth);
	  pile_rocks(x + 1, y - 1, tall, growth);
	}
	else {
	  pile_rocks(x - 1, y + 1, tall, growth);
	  pile_rocks(x + 1, y + 1, tall, growth);
	}
	printf("MAPMAKER: Cleaning up the rubble...\n");
	for (yc = 0; yc < ysize; yc++)
	  for (xc = 0; xc < xsize; xc++) 
		 if (map[xc][yc].terrain == 'E')
			map[xc][yc].terrain = '.';
		}
		else
	printf("MAPMAKER: Attempted to create hill on water hex. Aborted.\n");
	 }
	 printf("MAPMAKER: Automatic hill creation complete.\n");
  }
  else
	 printf("MAPMAKER: 0 hexes to seed. Automatic hill creation skipped.\n");

  tall = growth = 0;

  while (answer > 1 || answer < 0) {
	 printf("          Do you wish to seed more hexes manually? [y=1/n=0] ");
	 scanf("%d", &answer);
  }
  if (answer == 1) {
	 while (nument < 0 || nument > xsize * ysize) {
		printf("          How many hills will you raise? ");
		scanf("%d", &nument);
		if (nument > xsize * ysize)
	printf("MAPMAKER: That is more than the number of available hexes.");
	 }
	 for (i = 1;i <= nument; i++) {
		while (sx < 0 || sx >= xsize || sy < 0 || sy >= ysize) {
	printf("          Please enter a position for seed #%d: ", i);
	scanf("%d %d", &sx, &sy);
	if (map[sx][sy].height) {
	  printf("MAPMAKER: That is a water hex.");
	  sx = sy = -1;
	}
	else if (sx < 0 || sx >= xsize || sy < 0 || sy >= ysize)
	  printf("MAPMAKER: Please enter coordinates ranging from 0-%d and 0-%d\n", xsize - 1, ysize - 1);
		}
		printf("MAPMAKER: Starting hill at (%d, %d). What should the growth rate be?\n          [Enter 0 to abort this particular hill] ", sx, sy);
		scanf("%d", &growth);
		while (tall < 1 || tall > 9) {
	printf("          What should the initial height of this hill be? [1-9] ");
	scanf("%d", &tall);
		}
		if (growth > 0) {
	map[sx][sy].terrain = t;
	map[sx][sy].height = tall;
	pile_rocks(sx, sy + 1, tall, growth);
	pile_rocks(sx, sy - 1, tall, growth);
	pile_rocks(sx + 1, sy, tall, growth);
	pile_rocks(sx - 1, sy, tall, growth);
	if (x % 2) {
	  pile_rocks(sx + 1, sy - 1, tall, growth);
	  pile_rocks(sx - 1, sy - 1, tall, growth);
	}
	else {
	  pile_rocks(sx + 1, sy + 1, tall, growth);
	  pile_rocks(sx - 1, sy + 1, tall, growth);
	}
	printf("MAPMAKER: Hill raised.\n");
	printf("MAPMAKER: Cleaning up the rubble...\n");
	for (yc = 0; yc < ysize; yc++)
	  for (xc = 0; xc < xsize; xc++) 
		 if (map[xc][yc].terrain == 'E')
			map[xc][yc].terrain = '.';
		}
		sx = sy = -1;
	 }
  }
}

void make_plateau()
{
  printf("MAPMAKER: This feature not yet installed.");
}

void make_hill()
{
  printf("MAPMAKER: This feature not yet installed.");
}

void make_ridge()
{
  int dir, i, l, x, y, h;

  dir = l = h = -1;
  x = y = -1;
  while (x < 0 || x >= xsize || y < 0 || y >= ysize) {
	 printf("          Please enter the starting coordinates: ");
	 scanf("%d %d", &x, &y);
	 if (map[x][y].terrain == 't') {
		x = y = -1;
		printf("MAPMAKER: That's a water hex. Please pick a different point.");
	 }
  }
  printf("MAPMAKER: Building ridge at (%d, %d).\n", x, y);
  while (l < 1) {
	 printf("          How long (in hexes) should this ridge be? ");
	 scanf("%d", &l);
  }
  while (h < 1 || h > 9) {
	 printf("          How high should the ridge be? [1-9] ");
	 scanf("%d", &h);
  }
  while (dir != 0 && dir != 60 && dir != 120 && dir != 180 && dir != 240 && dir != 270 && dir != 300) {
	 printf("          The ridge can run: 0 degrees, 60 degrees, 90 degrees, 120 degrees,\n          180 degrees, 240 degrees, 270 degrees or 300 degrees.  Which\n          direction should it go? ");
	 scanf("%d", &dir);
  }
  
  map[x][y].height = h;
  for (i = 2; i <= l; i++) {
	 if (dir == 0)
		y--;
	 else if (dir == 60) {
		if (x % 2)
	y--;
		x++;
	 }
	 else if (dir == 90)
		x++;
	 else if (dir == 120) {
		if (!(x % 2))
	y++;
		x++;
	 }
	 else if (dir == 180)
		y++;
	 else if (dir == 240) {
		if (!(x % 2))
	y++;
		x--;
	 }
	 else if (dir == 270)
		x--;
	 else if (dir == 300) { 
		if (x % 2)
	y--;
		x--;
	 }
	 if (map[x][y].terrain == '~')
		printf("MAPMAKER: MAKE_RIDGE - Skipping water hex.");
	 else
		map[x][y].height = h;
  }
  printf("MAPMAKER: Ridge built.");
}

void set_alt(e)
int e;
{
  int answer;

  answer = 0;
  while(answer >= 0) {
	 printf("          What kind of feature would you like built?\n          (-1 to quit, 0 for list) ");
	 scanf("%d", &answer);
	 switch(answer) {
	 
	 case -1:
		break;
	 case 0:
		printf("MAPMAKER: The following elevation features are available:\n          1 - Plateau\n          2 - Hill\n          3 - Ridge.\n");
		break;
	 case 1:
		printf("MAPMAKER: Building plateau...\n");
		make_plateau();
		break;
	 case 2:
		printf("MAPMAKER: Building hill...\n");
		make_hill();
		break;
	 case 3:
		printf("MAPMAKER: Building ridge...\n");
		make_ridge();
		break;
	 default:
		printf("MAPMAKER: Option unknown.\n");
		break;
	 }
  }
}

void save_map()
{
  int xc, yc;
  char filename[30];
  FILE *rfptr;

  printf("Save this to which file? ");
  scanf("%sy", filename);
  
  if ((rfptr = fopen(filename, "w")) == NULL)
	 printf("MAPMAKER: The file could not be opened.\n");
  else {
	 fprintf(rfptr, "%d %d\n", xsize, ysize);
	 for (yc = 0; yc < ysize; yc++) {
		for (xc = 0; xc < xsize; xc++)
	fprintf(rfptr, "%c%d", map[xc][yc].terrain, map[xc][yc].height);
		fprintf(rfptr, "\n");
	 }
	 fprintf(rfptr, "-1\n-1 -1 -1 -1\n");
	 fclose(rfptr);
	 printf("MAPMAKER: Saved map to file '%s'.\n", filename);
  }
}

main(argc, argv)
int argc;
char *argv[];
{
  int rivers, lakes, woods, hills;
  int elevation;

  rivers = lakes = woods = hills = -1;
  elevation = -1;


  if (argc != 3)
	 printf("Syntax: mapmaker xsize ysize\n\n\ti.e. mapmaker 5 5\n");
  else {
	 xsize = atoi(argv[1]);
	 ysize = atoi(argv[2]);
	 if (xsize <= 0 || xsize > MAXSIZE || ysize <= 0 || ysize > MAXSIZE) {
		printf("You entered: mapmaker %s %s\nPlease make sure that you use values from 1-%d for your dimensions.\nAborting command.\n", argv[1], argv[2], MAXSIZE);
		return;
	 }
	 initialize_map();

	 while (rivers < 0 || lakes < 0) {
		printf("          How many major rivers would you like? ");
		scanf("%d", &rivers);
		printf("          What number of lakes should exist? ");
		scanf("%d", &lakes);
	 }
	 srand(clock());
	 if (rivers || lakes)
		set_water(rivers, lakes);

	 while (woods < 0 || woods > xsize * ysize) {
		printf("          Out of %d hexes, how many should I seed with the beginnings\n          of forests? ", xsize * ysize);
		scanf("%d", &woods);
	 }
	 srand(clock());
	 set_woods(woods);

	 while (hills < 0 || hills > xsize * ysize) {
		printf("          Out of %d hexes, how many should I seed with the beginnings\n          of mountains/hilly terrain? ", xsize * ysize);
		scanf("%d", &hills);
	 }
	 srand(clock());
	 set_hills(hills);

	 while (elevation < 0 || elevation > 1) {
		printf("          Do you wish to add other elevation features? [y=1/n=0] ");
		scanf("%d", &elevation);
	 }
	 srand(clock());
	 if (elevation)
		set_alt();

	 save_map();
  }
}





